{vs. 1.0  1/9/2015
Revilla, E. (in press). Individual and agent based models in population ecology and conservation biology.
In: Population Ecology in Practice: Underused, Misused, and Abused Methods.
Eds Murray DL, Sandercock B. John Wiley & Sons Ltd. ISBN/ISSN: 9780470674130}

unit exponential_unit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, TAGraph, TASeries, Forms, Controls, Graphics,
  Dialogs, StdCtrls, ExtCtrls;

type           {here you declare the data structure for individuals}
  PAgent=^Agent;
  Agent = record
    sex : string[1];
    Age : integer;
    Status : Shortint;
  end;

  { Texponential_Form }    //here the Form is declared, with its objects and associated procedures

  Texponential_Form = class(TForm)
    Chart1: TChart;
    Chart1LineSeries1: TLineSeries;
    Chart1LineSeries2: TLineSeries;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Exit_Button: TButton;
    Abort_Button: TButton;
    Edit1: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Panel1: TPanel;
    Run_Button: TButton;
    procedure Abort_ButtonClick(Sender: TObject);
    procedure Exit_ButtonClick(Sender: TObject);
    procedure Label6Click(Sender: TObject);
    procedure Run_ButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    Procedure Pop_dynamics;
  end;

var               {here you declare global variables}
  exponential_Form: Texponential_Form;
  Population : Tlist;           // the population is declared, just a contanier for the individuals
  Individual : PAgent;
  n_ini : integer;
  rep_prob: real;
  surv_prob: real;
  max_age:integer;
  populationsize: integer;
  males_available:string;
  n_sim: integer;
  current_sim:integer;
  max_pop_size: integer;
  pop_size: array[1..100] of integer;
  sum_pop_size: array[1..100] of integer;
  to_file_out: TextFile;

const             {here you declare constants}
  max_years = 100;
  min_rep_age = 3;
  file_name = 'output.txt';

implementation

{$R *.lfm}

{ Texponential_Form }


Procedure Startpopulation;
var
a:integer;
begin
  Population:=Tlist.create;
  with population do
  begin
    for a:=1 to n_ini do
    begin
      new(Individual);
      Individual^.age:=0;        {alternative: Individual^.age:=random(10);}
      if random<0.5 then Individual^.sex:= 'f' else Individual^.sex:= 'm';
      Individual^.status:=0;     {0 for non breeding age; 1 for breeding age}
      Population.add(Individual);
    end;
  end;
end;

Procedure Reproduction;
var
a:integer;
begin
  with population do
  begin
    populationsize:=population.count;
    if (populationsize>1) and (males_available='yes') then
    for a:=0 to populationsize-1 do
    begin
      Individual:=Items[a];
      if individual^.sex='f' then
      if individual^.status=1 then
      if random<rep_prob then
      begin
        New(Individual);
        Individual^.age:=0;
        if random<0.5 then Individual^.sex:= 'f' else Individual^.sex:= 'm';
        Individual^.status:=0;
        Population.add(Individual);
      end;
    end;
  end;
end;


Procedure survival;
var
a,b:integer;
die:string;
begin
  {survival part}
  with population do
  begin
    b:=-1;
    populationsize:=population.count;
    for a:=0 to populationsize-1 do  {the index in the list starts at 0}
    begin
      b:=b+1;
      individual:=items[b]; {we need a new index b to be able to delete the individuals from the list}
      die:='no';
      if individual^.age>max_age then die:='yes' else
        if random>surv_prob then die:='yes';            {random generates a random number; 0<=random<1}
      if die='yes' then
      begin
        delete(b);
        dispose(individual);
        b:=b-1;           {to keep the index right}
      end;
    end;
  end;

  {aging part}
  populationsize:=population.count;
  males_available:='no';
  with population do
  begin
    for a:=0 to populationsize-1 do
    begin
      Individual:=Items[a];
      Individual^.age:=Individual^.age+1;
      if individual^.age>min_rep_age then individual^.status:=1 else individual^.status:=0;
      if individual^.sex='m' then males_available:='yes';
    end;
  end;

end;


Procedure Texponential_Form.Pop_dynamics;
var
a:integer;

begin
  with population do
  begin
    for a:=1 to max_years do
    begin
      reproduction;
      survival;
      {ploting pop size}
      if max_pop_size<populationsize then max_pop_size:=populationsize;
      if max_pop_size>Chart1.extent.YMax then
      begin
        Chart1.extent.YMax:=max_pop_size;
        application.processmessages;
      end;
      pop_size[a]:=populationsize;
      sum_pop_size[a]:=sum_pop_size[a]+ populationsize;

      {plot trajectory}
      Chart1LineSeries1.addxy(a,populationsize);

    end;

  end;
end;


procedure Texponential_Form.Run_ButtonClick(Sender: TObject);
var
  a,b: integer;

begin
  randomize; {initialize the pseudorandom number generator}
  val(Edit1.text,n_ini);  {read parameter values from the form}
  val(Edit2.text,rep_prob);
  val(Edit3.text,surv_prob);
  val(Edit4.text,max_age);
  val(Edit5.text,n_sim);
  Chart1.extent.YMax:=250;
  AssignFile(to_file_out, file_name);
  rewrite(to_file_out); {create txt file for the output}

  for a:=1 to 100 do sum_pop_size[a]:=0;
  for current_sim:=1 to n_sim do
  begin

    max_pop_size:=0;
    Startpopulation; {call the procedure to initialize your population}
    Pop_dynamics;    {call the procedure to run the population dynamics}


    {plot population trayectories}
    Chart1LineSeries1.clear;
    Chart1LineSeries2.clear;
    for b:=1 to 100 do Chart1LineSeries1.addxy(b,pop_size[b]);
    for b:=1 to 100 do Chart1LineSeries2.addxy(b,sum_pop_size[b]/current_sim);

    {save the results to a text file}
    append(to_file_out);
    for b:=1 to 100 do writeln(to_file_out,current_sim,' ',b,' ',pop_size[b]);    {we save all simulations}
    if current_sim= n_sim then
      for b:=1 to 100 do writeln(to_file_out,'avg',' ',b,' ',sum_pop_size[b]/current_sim); {and the average of all simulations}
    CloseFile(to_file_out);

    application.processmessages;
  end;
end;

procedure Texponential_Form.Exit_ButtonClick(Sender: TObject);
begin
  close;
end;

procedure Texponential_Form.Label6Click(Sender: TObject);
begin

end;

procedure Texponential_Form.Abort_ButtonClick(Sender: TObject);
begin
  halt;
end;

end.

